@@ -5,6 +5,8 @@ from transformers import AutoTokenizer, AutoModelForCausalLM, pipeline
 
 import copy
 import pandas as pd
+import json
+import re
 
 
 def _ensure_pad_token(tok):
@@ -56,15 +58,32 @@ def _split_kwargs_simple(
     return pipeline_kwargs, generation_kwargs, model_kwargs, tokenizer_kwargs
 
 
+def _extract_json_block(text: str) -> dict:
+    """
+    Extrait le premier bloc JSON trouvé dans `text` et le parse en dict.
+    Retourne {} si rien de valide n'est trouvé.
+    """
+    try:
+        m = re.search(r"\{.*\}", text, re.DOTALL)
+        if m:
+            return json.loads(m.group(0))
+    except json.JSONDecodeError:
+        pass
+    return {}
+
+
 class Model:
     """Wrapper simple pour modèles de génération de texte (text-generation uniquement)."""
 
-    def __init__(self, name: str, task: str = "text-generation", **kwargs: Any):
+    def __init__(
+        self, name: str, task: str = "text-generation", prompt: str = "", **kwargs: Any
+    ):
         if task != "text-generation":
             raise ValueError("Only 'text-generation' is supported in this Model.")
 
         self.name = name
         self.task = task
+        self.prompt = prompt
 
         (
             self.pipeline_kwargs,
@@ -122,7 +141,7 @@ class Model:
         )
         print("[Model] Load completed.")
 
-    def run(self, dataset, prompt: str, output_col: str | None = None, **gen_overrides):
+    def run(self, dataset, output_col: str | None = None, **gen_overrides):
         if not hasattr(dataset, "data"):
             raise ValueError("[Model] No attribute 'data' found on dataset.")
 
@@ -146,9 +165,10 @@ class Model:
         infer_kwargs = {**self.generation_kwargs, **gen_overrides}
 
         outs = []
-        for txt in texts:
+        for i, txt in enumerate(texts, start=1):
+            print(f"\n===== Note {i}/{len(texts)} =====")
             print("[Model] Defining the prompt...")
-            inp = f"{prompt}\n\n{txt}".strip()
+            inp = f"{self.prompt}\n\n{txt}".strip()
 
             print("[Model] Generating...")
             result = self._pipe(inp, **infer_kwargs)
@@ -168,12 +188,10 @@ class Model:
 
         print(f"[Model] Generated {len(outs)} outputs.")
 
-        safe_name = self.name.replace("/", "_").replace("-", "_").replace(".", "_")
-        print(f"[Model] Processing {self.name} into {safe_name}...")
-
-        print(f"[Model] Determining output column name...")
+        safe_name = "Output"
+        print("[Model] Determining output column name...")
         if output_col is None:
-            output_col = f"{dataset.field}__{safe_name}"
+            output_col = f"{safe_name}"
 
         print(f"[Model] Initial output column name: {output_col}")
 
@@ -186,6 +204,15 @@ class Model:
         print(f"[Model] Final output column name: {output_col}")
         print(f"[Model] out is: {outs}")
 
+        # --- Post-traitement sûr : extraction JSON + colonnes structurées ---
+        parsed_series = df[output_col].apply(_extract_json_block)
+        df["Symptoms"] = parsed_series.apply(lambda d: d.get("Symptoms", []))
+        df["MedicalConclusion"] = parsed_series.apply(
+            lambda d: d.get("MedicalConclusion", [])
+        )
+        df["Treatments"] = parsed_series.apply(lambda d: d.get("Treatments", []))
+        df["Summary"] = parsed_series.apply(lambda d: d.get("Summary", ""))
+
         result_ds = copy.copy(dataset)
         result_ds.data = df
         result_ds.last_output_col = output_col
