@@ -7,6 +7,7 @@ import spacy
 import pandas as pd
 import numpy as np
 import warnings
+import json
 
 logger = logging.getLogger(__name__)
 
@@ -39,27 +40,27 @@ class Pipeline:
     # ------------------------- Orchestration -------------------------
 
     def apply(self):
-        logger.info("[Pipeline] Starting pipeline...")
+        print("[Pipeline] Starting pipeline...")
         self.homogenize()
         self.verify_QuickUMLS()
         self.verify_NLI()
-        logger.info("[Pipeline] Pipeline completed.")
+        print("[Pipeline] Pipeline completed.")
         return self.dataset_h
 
     # ------------------------- Homogénéisation -------------------------
 
     def homogenize(self):
-        logger.info("[Pipeline] Prompt for Homogenization:")
+        print("[Pipeline] Prompt for Homogenization:")
         if self.model_h.prompt:
-            logger.info(self.model_h.prompt)
+            print(self.model_h.prompt)
         else:
             self.model_h.prompt = self.build_prompt_h()
-            logger.info(self.model_h.prompt)
+            print(self.model_h.prompt)
 
-        logger.info("[Pipeline] Start Homogenization...")
+        print("[Pipeline] Start Homogenization...")
         out_h_col = f"{self.dataset.field}__h"
         self.dataset_h = self.model_h.run(self.dataset, output_col=out_h_col)
-        logger.info("[Pipeline] Homogenization completed.")
+        print("[Pipeline] Homogenization completed.")
 
     @staticmethod
     def build_prompt_h() -> str:
@@ -82,17 +83,79 @@ class Pipeline:
 
     def verify_QuickUMLS(self):
         # Placeholder: branchement futur à QuickUMLS
-        logger.info("[Pipeline] Starting QuickUMLS verification...")
-        logger.info("[Pipeline] QuickUMLS verification completed.")
+        print("[Pipeline] Starting QuickUMLS verification...")
+        df = self.dataset_h.data
+        print(f"[Pipeline] Processing {len(df)} rows...")
+        print(df.head(4))
+        print(list(df.columns))
+        print("[Pipeline] QuickUMLS verification completed.")
 
     def verify_NLI(self):
-        logger.info("[Pipeline] Starting NLI verification...")
-        self._ensure_spacy()
+        print("[Pipeline] Starting NLI verification...")
         self._ensure_nli()
-        logger.info({"id2label": self._id2label})
-        logger.info("[Pipeline] NLI verification completed.")
 
-    # ------------------------- NLI utils -------------------------
+        df = self.dataset_h.data
+        print(list(df.columns))
+        text_col = self.dataset.field
+        out_h_col = f"{self.dataset.field}__h"
+
+        # Ajout des colonnes de sortie si absentes
+        for col in ("nli_ent_mean", "nli_neu_mean", "nli_con_mean"):
+            if col not in df.columns:
+                df[col] = np.nan
+
+        for idx, row in df.iterrows():
+            text = str(row.get("full_note"))
+            print(f"\n===== Note {idx + 1}/{len(df)} =====")
+            print(f"[Pipeline] FN: {text[:60]}...")
+            if not text:
+                print(f"[Pipeline] Row {idx} has empty text, skipping NLI.")
+                continue
+
+            # Découper le texte en phrases
+            print(text)
+            premises = self.decouper_texte_en_phrases(text)
+            print(f"[Pipeline] Extracted {len(premises)} sentences.")
+            print(premises)
+            if not premises:
+                print(f"[Pipeline] Row {idx} has no sentences, skipping NLI.")
+                continue
+
+            # Extraire les hypothèses à partir du JSON homogénéisé
+            hypotheses = str(row.get("Summary"))
+            print(f"\n===== Note {idx + 1}/{len(df)} =====")
+            print(f"[Pipeline] Summ: {hypotheses[:60]}...")
+            if not hypotheses:
+                print(f"[Pipeline] Row {idx} has no hypotheses, skipping NLI.")
+                continue
+
+            hypotheses = self.decouper_texte_en_phrases(hypotheses)
+            print(f"[Pipeline] Extracted {len(hypotheses)} sentences.")
+            if not hypotheses:
+                print(f"[Pipeline] Row {idx} has no Summ sentences, skipping NLI.")
+                continue
+
+            # Calculer la matrice NLI (chaque phrase vs chaque hypothèse)
+            matrice = self.generer_table(
+                premises,
+                hypotheses,
+                lambda p, h: self.nli(p, h, return_probs=True),
+            )
+            print("[Pipeline] NLI matrix computed.")
+            print(matrice)
+
+            # Calculer la moyenne des meilleures hypothèses par phrase
+            avg = self.average(premises, hypotheses, matrice)
+
+            # Ajouter les moyennes dans le DataFrame
+            df.at[idx, "nli_ent_mean"] = avg["entailment"]
+            print("entailment: ", avg["entailment"])
+            df.at[idx, "nli_neu_mean"] = avg["neutral"]
+            print("neutral: ", avg["neutral"])
+            df.at[idx, "nli_con_mean"] = avg["contradiction"]
+            print("contradiction: ", avg["contradiction"])
+
+        print("[Pipeline] NLI verification completed.")
 
     def nli(self, premise: str, hypothesis: str, return_probs: bool = True) -> Dict:
         """Retourne la prédiction NLI et (optionnellement) les probabilités."""
@@ -132,31 +195,22 @@ class Pipeline:
     # ------------------------- spaCy utils -------------------------
 
     def decouper_texte_en_phrases(self, texte: str) -> List[str]:
-        nlp = self._ensure_spacy()
+        nlp = spacy.load("en_core_web_sm")
         doc = nlp(texte)
         return [sent.text.strip() for sent in doc.sents]
 
-    def _ensure_spacy(self):
-        if self._nlp is None:
-            try:
-                self._nlp = spacy.load("en_core_web_sm")
-            except OSError:
-                from spacy.cli import download
-
-                logger.info("Downloading spaCy model: en_core_web_sm")
-                download("en_core_web_sm")
-                self._nlp = spacy.load("en_core_web_sm")
-        return self._nlp
-
     # ------------------------- NLI model load -------------------------
 
     def _ensure_nli(self):
-        if self._tok is None or self._clf is None or self._id2label is None:
-            self._tok = AutoTokenizer.from_pretrained(self.NLI_MODEL_NAME)
-            self._clf = AutoModelForSequenceClassification.from_pretrained(
-                self.NLI_MODEL_NAME
-            ).to(self.device)
-            self._id2label = self._clf.config.id2label
+
+        self.tok = AutoTokenizer.from_pretrained(self.NLI_MODEL_NAME)
+        self.clf = AutoModelForSequenceClassification.from_pretrained(
+            self.NLI_MODEL_NAME
+        )
+
+        self.device = "cuda" if torch.cuda.is_available() else "cpu"
+        self.clf = self.clf.to(self.device)
+        self._id2label = self._clf.config.id2label
         return self._tok, self._clf, self._id2label
 
     # ------------------------- Tableaux & métriques -------------------------
@@ -174,7 +228,6 @@ class Pipeline:
     def _prettier(res: Dict) -> Dict:
         """Nettoie/valide une cellule { 'probs': {...} } -> retourne le dict des probs."""
         probs = (res or {}).get("probs", {})
-        # Optionnel: assurer la présence des trois clés
         for k in ("entailment", "neutral", "contradiction"):
             probs.setdefault(k, None)
         return probs
@@ -204,7 +257,7 @@ class Pipeline:
         mean_best_neu = float(np.nanmean(best_neu_vals)) if best_neu_vals else None
         mean_best_con = float(np.nanmean(best_con_vals)) if best_con_vals else None
 
-        logger.info(
+        print(
             "Moyennes — entailment=%s, neutral=%s, contradiction=%s",
             mean_best_ent,
             mean_best_neu,
